@@ -6,13 +6,13 @@ from aignostics import platform
 
 # initialize the client
 client = platform.Client()
-# create application run
+# submit application run
 # for details, see the IPython or Marimo notebooks for a detailed explanation of the payload
-application_run = client.runs.create(
-    application_version="two-task-dummy:v0.35.0",
+application_run = client.runs.submit(
+    application_id="two-task-dummy",
     items=[
         platform.InputItem(
-            reference="1",
+            external_id="1",
             input_artifacts=[
                 platform.InputArtifact(
                     name="user_slide",
@@ -20,10 +20,10 @@ application_run = client.runs.create(
                         "gs://aignx-storage-service-dev/sample_data_formatted/9375e3ed-28d2-4cf3-9fb9-8df9d11a6627.tiff"
                     ),
                     metadata={
-                        "checksum_crc32c": "N+LWCg==",
-                        "base_mpp": 0.46499982,
-                        "width": 3728,
-                        "height": 3640,
+                        "checksum_base64_crc32c": "N+LWCg==",
+                        "resolution_mpp": 0.46499982,
+                        "width_px": 3728,
+                        "height_px": 3640,
                     },
                 )
             ],
