@@ -1,12 +1,11 @@
 """Application module."""
 
 from ._cli import cli
+from ._models import DownloadProgress, DownloadProgressState
 from ._service import Service
+from ._settings import Settings
 
-__all__ = [
-    "Service",
-    "cli",
-]
+__all__ = ["DownloadProgress", "DownloadProgressState", "Service", "Settings", "cli"]
 
 from importlib.util import find_spec
 
