@@ -3,76 +3,111 @@
 from importlib.util import find_spec
 from multiprocessing import Manager
 from pathlib import Path
-from typing import Any
+from typing import TYPE_CHECKING, Any
 from urllib.parse import quote
 
+import humanize
+from aiopath import AsyncPath
+from loguru import logger
+from nicegui import (
+    app,
+    ui,  # noq
+)
 from nicegui import run as nicegui_run
-from nicegui import ui  # noq
-from showinfm.showinfm import show_in_file_manager
 
-from aignostics.platform import ApplicationRunStatus, ItemStatus
-from aignostics.utils import GUILocalFilePicker, get_logger, get_user_data_directory
+from aignostics.platform import ItemOutput, ItemState, RunState
+from aignostics.third_party.showinfm.showinfm import show_in_file_manager
+from aignostics.utils import GUILocalFilePicker, get_user_data_directory
 
-from .._service import DownloadProgressState, Service  # noqa: TID252
+if TYPE_CHECKING:
+    from aignostics.platform import UserInfo
+
+from .._models import DownloadProgressState  # noqa: TID252
+from .._service import Service  # noqa: TID252
 from .._utils import get_mime_type_for_artifact  # noqa: TID252
 from ._frame import _frame
 from ._utils import (
     mime_type_to_icon,
-    run_item_status_to_icon_and_color,
+    run_item_status_and_termination_reason_to_icon_and_color,
     run_status_to_icon_and_color,
 )
 
-logger = get_logger(__name__)
-
-WIDTH_100 = "width: 100%"
 WIDTH_1200px = "width: 1200px; max-width: none"
 
 service = Service()
 
 
-async def _page_application_run_describe(application_run_id: str) -> None:  # noqa: C901, PLR0912, PLR0915
+async def _page_application_run_describe(run_id: str) -> None:  # noqa: C901, PLR0912, PLR0914, PLR0915
     """Describe Application.
 
     Args:
-        application_run_id (str): The ID of the application run to describe.
+        run_id (str): The ID of the application run to describe.
     """
     import pandas as pd  # noqa: PLC0415
 
     if find_spec("ijson"):
         from aignostics.qupath import Service as QuPathService  # noqa: PLC0415
 
+    ui.add_head_html("""
+        <style>
+        /* Force text wrapping in code blocks */
+        .nicegui-code pre {
+            white-space: pre-wrap !important;
+            word-wrap: break-word !important;
+            overflow-wrap: break-word !important;
+            max-width: 100% !important;
+        }
+        /* Remove padding from expansion items to make full use of space */
+        .q-expansion-item .q-item {
+            padding-left: 0 !important;
+        }
+        .q-expansion-item .q-expansion-item__content {
+            padding: 0 !important;
+        }
+        </style>
+    """)
+
     spinner = ui.spinner(size="xl").classes("fixed inset-0 m-auto")
-    run = await nicegui_run.io_bound(service.application_run, application_run_id)
+    run = await nicegui_run.io_bound(service.application_run, run_id)
     spinner.set_visibility(False)
-    run_data = run.details()
+    run_data = run.details() if run else None
 
     if run and run_data:
-        icon, color = run_status_to_icon_and_color(run_data.status.value)
+        icon, color = run_status_to_icon_and_color(
+            run_data.state.value,
+            run_data.termination_reason,
+            run_data.statistics.item_count,
+            run_data.statistics.item_succeeded_count,
+        )
         await _frame(
             navigation_title=(
-                f"Run of {run_data.application_version_id} "
-                f"on {run_data.triggered_at.astimezone().strftime('%m-%d %H:%M')}"
+                f"Run of {run_data.application_id} ({run_data.version_number}) on "
+                f"{run_data.submitted_at.astimezone().strftime('%m-%d %H:%M')}"
             ),
             navigation_icon=icon,
             navigation_icon_color=color,
-            navigation_icon_tooltip=f"Run {run_data.application_run_id}, status {run_data.status.value.upper()}",
+            navigation_icon_tooltip=f"Run {run_data.run_id}, status {run_data.state.value.upper()}",
             left_sidebar=True,
-            args={"application_run_id": application_run_id},
+            args={"run_id": run_id},
         )
     else:
         await _frame(
-            navigation_title=f"Run {application_run_id}",
+            navigation_title=f"Run {run_id}",
             navigation_icon="bug_report",
             navigation_icon_color="negative",
             navigation_icon_tooltip="Could not load run data",
             left_sidebar=True,
-            args={"application_run_id": application_run_id},
+            args={"run_id": run_id},
         )
 
     if run is None:
-        ui.label(f"Failed to get run '{application_run_id}'").mark("LABEL_ERROR")  # type: ignore[unreachable]
+        ui.label(f"Failed to get run '{run_id}'").mark("LABEL_ERROR")  # type: ignore[unreachable]
         return
 
+    # Forward declaration of UI buttons that will be defined later
+    cancel_button: ui.button
+    delete_button: ui.button
+
     async def _cancel(run_id: str) -> bool:
         """Cancel the application run.
 
@@ -91,12 +126,36 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
             ui.navigate.reload()
             ui.notify("Application run cancelled!", type="positive")
             return True
-        except Exception as e:  # noqa: BLE001
+        except Exception as e:
             cancel_button.enable()
             cancel_button.props(remove="loading")
             ui.notify(f"Failed to cancel application run: {e}.", type="warning")
             return False
 
+    async def _delete(run_id: str) -> bool:
+        """Delete the application run results and navigate to the main page.
+
+        Args:
+            run_id (str): The ID of the run to cancel.
+
+        Returns:
+            bool: True if the run was cancelled, False otherwise.
+        """
+        ui.notify(f"Deleting results of application run with id '{run_id}' ...", type="info")
+        try:
+            delete_button.disable()
+            delete_button.props(add="loading")
+            await nicegui_run.io_bound(service.application_run_delete, run_id)
+            delete_button.props(remove="loading")
+            ui.navigate.to("/")
+            ui.notify("Application run deleted!", type="positive")
+            return True
+        except Exception as e:
+            delete_button.enable()
+            delete_button.props(remove="loading")
+            ui.notify(f"Failed to delete results of application run: {e}.", type="warning")
+            return False
+
     @ui.refreshable
     def download_run_dialog_content(qupath_project: bool = False, marimo: bool = False) -> None:  # noqa: C901, PLR0915
         if qupath_project:
@@ -127,13 +186,14 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
         with ui.row().classes("w-full"):
 
             async def _select_download_destination() -> None:
-                result = await GUILocalFilePicker(str(Path.home()), multiple=False)  # type: ignore[misc]
+                result = await GUILocalFilePicker(str(Path(await AsyncPath.home())), multiple=False)  # type: ignore[misc]
                 if result and len(result) > 0:
-                    folder_path = Path(result[0])
-                    if folder_path.is_dir():
+                    folder_path = AsyncPath(result[0])
+                    if await folder_path.is_dir():
                         selected_folder.value = str(folder_path)
                     else:
                         selected_folder.value = str(folder_path.parent)
+                    ui.notify(f"Using custom directory: {selected_folder.value}", type="info")
                     download_button.enable()
                 else:
                     ui.notify("No folder selected", type="warning")
@@ -141,6 +201,7 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
             async def _select_data() -> None:  # noqa: RUF029
                 """Open a file picker dialog and show notifier when closed again."""
                 selected_folder.value = str(get_user_data_directory("results"))
+                ui.notify("Using Launchpad results directory", type="info")
                 download_button.enable()
 
             with ui.row().classes("w-full"):
@@ -168,22 +229,32 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                 ui.notify("Please select a folder first", type="warning")
                 return
 
-            ui.notify("Downloading  ...", type="info")
+            ui.notify("Downloading ...", type="info")
             progress_queue = Manager().Queue()
 
-            def update_download_progress() -> None:
+            def update_download_progress() -> None:  # noqa: C901, PLR0912
                 """Update the progress indicator with values from the queue."""
                 while not progress_queue.empty():
                     progress = progress_queue.get()
-                    download_item_status.set_text(
-                        progress.status
-                        if progress.status is not DownloadProgressState.DOWNLOADING
-                        or progress.total_artifact_index is None
-                        else (
+                    # Determine status text based on progress state
+                    if progress.status is DownloadProgressState.DOWNLOADING_INPUT:
+                        status_text = (
+                            f"Downloading input slide {progress.item_index + 1} of {progress.item_count}"
+                            if progress.item_index is not None and progress.item_count
+                            else "Downloading input slide ..."
+                        )
+                    elif (
+                        progress.status is DownloadProgressState.DOWNLOADING
+                        and progress.total_artifact_index is not None
+                    ):
+                        status_text = (
                             f"Downloading artifact {progress.total_artifact_index + 1} "
                             f"of {progress.total_artifact_count}"
                         )
-                    )
+                    else:
+                        status_text = progress.status
+
+                    download_item_status.set_text(status_text)
                     download_item_status.set_visibility(True)
                     download_item_progress.set_value(progress.item_progress_normalized)
                     download_artifact_progress.set_value(progress.artifact_progress_normalized)
@@ -191,7 +262,13 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                         download_artifact_status.set_visibility(False)
                         download_item_progress.set_visibility(False)
                         download_artifact_progress.set_visibility(False)
-                    if progress.status is DownloadProgressState.DOWNLOADING:
+                    elif progress.status is DownloadProgressState.DOWNLOADING_INPUT:
+                        if progress.input_slide_path:
+                            download_artifact_status.set_text(f"Input: {progress.input_slide_path.name}")
+                        download_artifact_status.set_visibility(True)
+                        download_item_progress.set_visibility(True)
+                        download_artifact_progress.set_visibility(True)
+                    elif progress.status is DownloadProgressState.DOWNLOADING:
                         if progress.artifact_path:
                             download_artifact_status.set_text(str(progress.artifact_path))
                         download_artifact_status.set_visibility(True)
@@ -231,7 +308,7 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                 download_button.props(add="loading")
                 results_folder = await nicegui_run.cpu_bound(
                     Service.application_run_download_static,
-                    run_id=run.application_run_id,
+                    run_id=run.run_id,
                     destination_directory=Path(selected_folder.value),
                     wait_for_completion=True,
                     qupath_project=qupath_project,
@@ -302,7 +379,7 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
         if url:
             try:
                 csv_df = pd.read_csv(url, comment="#")
-            except Exception as e:  # noqa: BLE001
+            except Exception as e:
                 ui.notify(f"Failed to load CSV: {e!s}", type="negative")
                 csv_df = pd.DataFrame()  # Empty dataframe as fallback
             ui.aggrid.from_pandas(csv_df)
@@ -330,8 +407,8 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                     "mainMenuBar": False,
                     "navigationBar": True,
                     "statusBar": False,
-                }).style(WIDTH_100)
-            except Exception as e:  # noqa: BLE001
+                }).classes("full-width")
+            except Exception as e:
                 ui.notify(f"Failed to render metadata: {e!s}", type="negative")
 
     with ui.dialog() as metadata_dialog, ui.card().style(WIDTH_1200px):
@@ -352,7 +429,7 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
             try:
                 with ui.scroll_area().classes("w-full h-[calc(100vh-2rem)]"):
                     ui.image("/tiff?url=" + quote(url))
-            except Exception as e:  # noqa: BLE001
+            except Exception as e:
                 ui.notify(f"Failed to load CSV: {e!s}", type="negative")
 
     with ui.dialog() as tiff_view_dialog, ui.card().style(WIDTH_1200px):
@@ -371,6 +448,33 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
         tiff_view_dialog_content.refresh(title=title, url=url)
         tiff_view_dialog.open()
 
+    @ui.refreshable
+    def custom_metadata_dialog_content(title: str | None, custom_metadata: str | None) -> None:
+        if title:
+            ui.label(title).classes("text-h5")
+        if custom_metadata:
+            try:
+                ui.json_editor({
+                    "content": {"json": custom_metadata},
+                    "mode": "tree",
+                    "readOnly": True,
+                    "mainMenuBar": False,
+                    "navigationBar": True,
+                    "statusBar": False,
+                }).classes("full-width")
+            except Exception as e:
+                ui.notify(f"Failed to render metadata: {e!s}", type="negative")
+
+    with ui.dialog() as custom_metadata_dialog, ui.card().style(WIDTH_1200px):
+        custom_metadata_dialog_content(title=None, custom_metadata=None)
+        with ui.row(align_items="end").classes("w-full"), ui.column(align_items="end").classes("w-full"):
+            ui.button("Close", on_click=custom_metadata_dialog.close)
+
+    def custom_metadata_dialog_open(title: str, custom_metadata: dict[str, Any]) -> None:
+        """Open the Custom Metadata dialog."""
+        custom_metadata_dialog_content.refresh(title=title, custom_metadata=custom_metadata)
+        custom_metadata_dialog.open()
+
     async def open_qupath(
         project: Path | None = None, image: Path | str | None = None, button: ui.button | None = None
     ) -> None:
@@ -383,7 +487,7 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
             pid = await nicegui_run.cpu_bound(QuPathService.execute_qupath, project=project, image=image)
             if pid:
                 message = f"QuPath opened successfully with process id '{pid}'."
-                logger.info(message)
+                logger.debug(message)
                 ui.notify(message, type="positive")
             else:
                 message = "Failed to launch QuPath."
@@ -401,24 +505,87 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
         if button:
             button.disable()
             button.props(add="loading")
-        ui.navigate.to(f"/notebook/{run.application_run_id}?results_folder={quote(results_folder.as_posix())}")
+        ui.navigate.to(f"/notebook/{run.run_id}?results_folder={quote(results_folder.as_posix())}")
         ui.navigate.reload()  # TODO(Helmut): Find out why this workaround works. Was just a hunch ...
 
     if run_data:  # noqa: PLR1702
         with ui.row().classes("w-full justify-center"):
-            with ui.expansion(text=f"Run {run.application_run_id}"):
-                ui.markdown(
+            expansion = ui.expansion(text=f"Run {run.run_id}", icon="info")
+            expansion.on_value_change(
+                lambda e: expansion.classes(add="w-full" if e.value else "", remove="w-full" if not e.value else "")
+            )
+            with expansion:
+                # Display run metadata, including duration if possible, using humanize
+
+                submitted_at = run_data.submitted_at.astimezone()
+                terminated_at = run_data.terminated_at.astimezone() if run_data.terminated_at else None
+                if submitted_at and terminated_at:
+                    duration_seconds = (terminated_at - submitted_at).total_seconds()
+                    duration_str = humanize.precisedelta(duration_seconds, format="%0.0f")
+                else:
+                    duration_str = "N/A"
+
+                if run_data.state is RunState.TERMINATED and run_data.termination_reason:
+                    status_str = f"{run_data.state.value} ({run_data.termination_reason.name})"
+                else:
+                    status_str = f"{run_data.state.value}"
+
+                ui.code(
                     f"""
-                    * Run ID: {run_data.application_run_id}
-                    * Application Version: {run_data.application_version_id}
-                    * Triggered On: {run_data.triggered_at.astimezone().strftime("%m-%d %H:%M")}
-                    * Triggered by: {run_data.triggered_by}
-                    * Organization: {run_data.organization_id}
-                    """
-                )
+                    * Run ID: {run_data.run_id}
+                    * Application: {run_data.application_id} ({run_data.version_number})
+                    * Status: {status_str}
+                    * Output: {run_data.output.name}
+                        - {run_data.statistics.item_count} items
+                        - {run_data.statistics.item_pending_count} pending
+                        - {run_data.statistics.item_processing_count} processing
+                        - {run_data.statistics.item_skipped_count} skipped
+                        - {run_data.statistics.item_succeeded_count} succeeded
+                        - {run_data.statistics.item_user_error_count} user errors
+                        - {run_data.statistics.item_system_error_count} system errors
+                    * Submitted: {submitted_at.strftime("%m-%d %H:%M")} ({run_data.submitted_by})
+                    * Terminated: {terminated_at.strftime("%m-%d %H:%M") if terminated_at else "N/A"} ({duration_str})
+                    * Error: {run_data.error_message or "N/A"} ({run_data.error_code or "N/A"})
+                    """,
+                    language="markdown",
+                ).classes("full-width").mark("CODE_RUN_METADATA")
+                user_info: UserInfo | None = app.storage.tab.get("user_info", None)
+                if run_data.custom_metadata:
+                    is_editable = user_info and user_info.role in {"admin", "super_admin"}
+                    properties = {
+                        "content": {"json": run_data.custom_metadata},
+                        "mode": "tree",
+                        "readOnly": not is_editable,
+                        "mainMenuBar": True,
+                        "navigationBar": False,
+                        "statusBar": False,
+                    }
+
+                    async def handle_metadata_change(e: Any) -> None:  # noqa: ANN401
+                        """Handle changes to the custom metadata and update the run."""
+                        if not is_editable:
+                            return
+                        try:
+                            # Extract the new metadata from the event's content attribute
+                            new_metadata = e.content.get("json") if hasattr(e, "content") else None
+                            if new_metadata:
+                                ui.notify("Updating custom metadata...", type="info")
+                                await nicegui_run.io_bound(
+                                    Service.application_run_update_custom_metadata_static,
+                                    run_id=run_id,
+                                    custom_metadata=new_metadata,
+                                )
+                                ui.notify("Custom metadata updated successfully!", type="positive")
+                                ui.navigate.reload()
+                        except Exception as ex:
+                            ui.notify(f"Failed to update custom metadata: {ex!s}", type="negative")
+
+                    ui.json_editor(properties, on_change=handle_metadata_change).classes("full-width").mark(
+                        "JSON_EDITOR_CUSTOM_METADATA"
+                    )
             ui.space()
             with ui.row().classes("justify-end"):
-                if run_data.status.value == ApplicationRunStatus.COMPLETED:
+                if run_data.state.value == RunState.TERMINATED and run_data.statistics.item_succeeded_count > 0:
                     with ui.button_group().props("push"):
                         with (
                             ui.button("Download", icon="cloud_download", on_click=lambda _: download_run_dialog_open())
@@ -448,14 +615,38 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                                 .props("push")
                             ):
                                 ui.tooltip("Open results in Python Notebook served by Marimo")
-                if run_data.status.value == ApplicationRunStatus.RUNNING:
+
+                if run_data.state.value in {RunState.PENDING, RunState.PROCESSING}:
                     cancel_button = ui.button(
                         "Cancel",
                         color="red",
-                        on_click=lambda: _cancel(run.application_run_id),
+                        on_click=lambda: _cancel(run.run_id),
                         icon="cancel",
                     ).mark("BUTTON_APPLICATION_RUN_CANCEL")
 
+                if run_data:
+                    delete_button = ui.button(
+                        "Delete",
+                        color="red",
+                        on_click=lambda: _delete(run.run_id),
+                        icon="delete",
+                    ).mark("BUTTON_APPLICATION_RUN_RESULT_DELETE")
+
+        note = run_data.custom_metadata.get("sdk", {}).get("note") if run_data.custom_metadata else None
+        if note:
+            with ui.card().classes("full-width bg-aignostics-light"):
+                ui.label("Note:").classes("text-italic text-sm text-gray-500")
+                ui.label(str(note)).classes("-mt-4")
+
+        tags = run_data.custom_metadata.get("sdk", {}).get("tags") if run_data.custom_metadata else None
+        if tags and len(tags):
+            with ui.row().classes("gap-1 -mt-2 full-width"):
+                for tag in tags[:20]:
+                    ui.chip(
+                        tag,
+                        on_click=lambda t=tag: ui.navigate.to(f"/?query={quote(str(t))}"),
+                    ).props("small outlined clickable").classes("bg-white text-black")
+
         with ui.list().classes("full-width"):
             results = list(run.results())
             if not results:
@@ -464,35 +655,36 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                     ui.html(
                         '<dotlottie-player src="/application_assets/empty.lottie" '
                         'background="transparent" speed="1" style="width: 700px; height: 700px" '
-                        'direction="1" playMode="normal" loop autoplay></dotlottie-player>'
+                        'direction="1" playMode="normal" loop autoplay></dotlottie-player>',
+                        sanitize=False,
                     )
                     ui.space()
                 return
             for item in results:
-                with ui.item().classes("h-96").props("clickable"):
+                with ui.item().classes("h-96 px-0").props("clickable"):
                     with (
                         ui.item_section().classes("h-full"),
                         ui.card().tight().classes("h-full"),
                         ui.row().classes("w-full"),
                     ):
-                        image_file: Path | None = Path(item.reference).resolve()
-                        if image_file and image_file.is_file():
+                        image_file: AsyncPath | None = await AsyncPath(item.external_id).resolve()
+                        if image_file and await image_file.is_file():
                             image_url = "/thumbnail?source=" + quote(image_file.as_posix())
                         else:
                             image_file = None
                             image_url = "/application_assets/image-not-found.png"
                         ui.image(image_url).classes("object-contain absolute-center max-h-full")
-                        icon, color = run_item_status_to_icon_and_color(item.status.value)
+                        icon, color = run_item_status_and_termination_reason_to_icon_and_color(
+                            item.state.value, item.termination_reason
+                        )
                         with ui.row().classes("justify-center w-full"):
                             with ui.icon(icon, color=color).classes("text-4xl pl-2 pt-1").props("floating"):
-                                if item.error:
-                                    ui.tooltip(
-                                        f"Item {item.item_id}, status {item.status.value.upper()}, error: {item.error}"
-                                    )
-                                else:
-                                    ui.tooltip(f"Item {item.item_id}, status {item.status.value.upper()}")
+                                tooltip = f"Item {item.item_id}, status {item.state.value.upper()}"
+                                if item.termination_reason:
+                                    tooltip += f" ({item.termination_reason})"
+                                ui.tooltip(tooltip)
                             ui.space()
-                            with ui.button_group().props():
+                            with ui.button_group():
                                 if find_spec("ijson") and QuPathService.is_qupath_installed():
                                     with ui.button(
                                         icon="zoom_in",
@@ -504,10 +696,20 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                                             )
                                         )
                                         ui.tooltip("Open in QuPath")
+                                if item.custom_metadata:
+                                    with ui.button(
+                                        icon="info",
+                                        on_click=lambda _,
+                                        custom_metadata=item.custom_metadata,
+                                        external_id=item.external_id: custom_metadata_dialog_open(
+                                            title=f"Custom Metadata of item {external_id} ",
+                                            custom_metadata=custom_metadata,
+                                        ),
+                                    ).props("floating"):
+                                        ui.tooltip("Show custom metadata")
                                 if image_file:
                                     with ui.button(
                                         icon="folder_open",
-                                        color="primary",
                                         on_click=lambda _, image_file=image_file: show_in_file_manager(
                                             str(image_file.parent)
                                         ),
@@ -516,76 +718,85 @@ async def _page_application_run_describe(application_run_id: str) -> None:  # no
                         with ui.row().classes(
                             "absolute-bottom h-32 bg-indigo-700 bg-opacity-80 content-center w-full p-4"
                         ):
-                            ui.label(item.reference).classes(
+                            ui.label(item.external_id).classes(
                                 "text-center break-all text-white font-semibold text-shadow-lg/30"
                             )
-                    if item.status is ItemStatus.SUCCEEDED:
-                        with ui.item_section().classes("w-full"):
-                            if item.status is ItemStatus.SUCCEEDED and item.output_artifacts:
-                                with ui.scroll_area().classes("h-full").style("padding: 0"):
-                                    for artifact in sorted(item.output_artifacts, key=lambda a: str(a.name)):
-                                        mime_type = get_mime_type_for_artifact(artifact)
-                                        with ui.expansion(
-                                            str(artifact.name),
-                                            icon=mime_type_to_icon(mime_type),
-                                            group="artifacts",
-                                        ).classes("w-full"):
-                                            if artifact.download_url:
-                                                url = artifact.download_url
-                                                title = artifact.name
-                                                metadata = artifact.metadata
-                                                with ui.button_group():
-                                                    if mime_type == "image/tiff":
-                                                        ui.button(
-                                                            "Preview",
-                                                            icon=mime_type_to_icon(mime_type),
-                                                            on_click=lambda _, url=url, title=title: tiff_dialog_open(
-                                                                title, url
-                                                            ),
-                                                        )
-                                                    if mime_type == "text/csv":
-                                                        ui.button(
-                                                            "Preview",
-                                                            icon=mime_type_to_icon(mime_type),
-                                                            on_click=lambda _, url=url, title=title: csv_dialog_open(
-                                                                title, url
-                                                            ),
-                                                        )
-                                                    if url:
-                                                        ui.button(
-                                                            text="Download",
-                                                            icon="cloud_download",
-                                                            on_click=lambda _, url=url: ui.navigate.to(
-                                                                url, new_tab=True
-                                                            ),
-                                                        )
-                                                    if metadata:
-                                                        ui.button(
-                                                            text="Schema",
-                                                            icon="schema",
-                                                            on_click=lambda _,
-                                                            title=title,
-                                                            metadata=metadata: metadata_dialog_open(title, metadata),
-                                                        )
-                    elif item.status in {
-                        ItemStatus.PENDING,
-                        ItemStatus.ERROR_USER,
-                        ItemStatus.ERROR_SYSTEM,
-                        ItemStatus.CANCELED_USER,
-                        ItemStatus.CANCELED_SYSTEM,
-                    }:
+                    if item.output is ItemOutput.FULL:
+                        with ui.item_section().classes("w-full"), ui.scroll_area().classes("h-full p-0"):
+                            for artifact in sorted(item.output_artifacts, key=lambda a: str(a.name)):
+                                mime_type = get_mime_type_for_artifact(artifact)
+                                with ui.expansion(
+                                    str(artifact.name),
+                                    icon=mime_type_to_icon(mime_type),
+                                    group="artifacts",
+                                ).classes("w-full"):
+                                    if artifact.download_url:
+                                        url = artifact.download_url
+                                        title = artifact.name
+                                        metadata = artifact.metadata
+                                        with ui.button_group():
+                                            if mime_type == "image/tiff":
+                                                ui.button(
+                                                    "Preview",
+                                                    icon=mime_type_to_icon(mime_type),
+                                                    on_click=lambda _, url=url, title=title: tiff_dialog_open(
+                                                        title, url
+                                                    ),
+                                                )
+                                            if mime_type == "text/csv":
+                                                ui.button(
+                                                    "Preview",
+                                                    icon=mime_type_to_icon(mime_type),
+                                                    on_click=lambda _, url=url, title=title: csv_dialog_open(
+                                                        title, url
+                                                    ),
+                                                )
+                                            if url:
+                                                ui.button(
+                                                    text="Download",
+                                                    icon="cloud_download",
+                                                    on_click=lambda _, url=url: ui.navigate.to(url, new_tab=True),
+                                                )
+                                            if metadata:
+                                                ui.button(
+                                                    text="Schema",
+                                                    icon="schema",
+                                                    on_click=lambda _,
+                                                    title=title,
+                                                    metadata=metadata: metadata_dialog_open(title, metadata),
+                                                )
+                    elif item.state is ItemState.TERMINATED:
+                        if item.error_message:
+                            with (
+                                ui.row()
+                                .classes("w-1/2 justify-start items-start content-start ml-4")
+                                .style("max-width: 50%;")
+                            ):
+                                ui.code(
+                                    f"Error: {item.error_message}, code: {item.error_code or 'N/A'}",
+                                    language="markdown",
+                                ).classes("ml-8").style("width: 100%; max-width: 100%;")
+                        else:
+                            with ui.row().classes("w-1/2 justify-center content-center"):
+                                ui.space()
+                                ui.html(
+                                    '<dotlottie-player src="/application_assets/error.lottie" '
+                                    'background="transparent" speed="1" style="width: 300px; height: 300px" '
+                                    'direction="1" playMode="normal" loop autoplay></dotlottie-player>',
+                                    sanitize=False,
+                                )
+                                ui.space()
+                    else:
                         with ui.row().classes("w-1/2 justify-center content-center"):
                             ui.space()
                             animation_file = {
-                                ItemStatus.PENDING: "pending.lottie",
-                                ItemStatus.ERROR_USER: "error.lottie",
-                                ItemStatus.ERROR_SYSTEM: "error.lottie",
-                                ItemStatus.CANCELED_USER: "canceled.lottie",
-                                ItemStatus.CANCELED_SYSTEM: "canceled.lottie",
-                            }[item.status]
+                                ItemState.PENDING: "pending.lottie",
+                                ItemState.PROCESSING: "processing.lottie",  # TODO(Helmut): Different icon
+                            }[item.state]
                             ui.html(
                                 f'<dotlottie-player src="/application_assets/{animation_file}" '
                                 'background="transparent" speed="1" style="width: 300px; height: 300px" '
-                                'direction="1" playMode="normal" loop autoplay></dotlottie-player>'
+                                'direction="1" playMode="normal" loop autoplay></dotlottie-player>',
+                                sanitize=False,
                             )
                             ui.space()
