@@ -6,11 +6,9 @@ __all__ = []
 
 from ._cli import cli
 from ._service import Service
+from ._settings import Settings
 
-__all__ += [
-    "Service",
-    "cli",
-]
+__all__ += ["Service", "Settings", "cli"]
 
 # advertise PageBuilder to enable auto-discovery
 if find_spec("nicegui"):
