@@ -17,20 +17,24 @@ from pathlib import Path
 from typing import Any
 from urllib.parse import urlparse, urlsplit
 
-import appdirs
 import ijson
+import platformdirs
 import psutil
 import requests
+from loguru import logger
 from packaging.version import Version
 from psutil import Process, wait_procs
 from pydantic import BaseModel, computed_field
 
-from aignostics.utils import UNHIDE_SENSITIVE_INFO, BaseService, Health, __project_name__, get_logger
+from aignostics.utils import (
+    SUBPROCESS_CREATION_FLAGS,
+    BaseService,
+    Health,
+    __project_name__,
+)
 
 from ._settings import Settings
 
-logger = get_logger(__name__)
-
 QUPATH_VERSION = "0.6.0-rc5"
 DOWNLOAD_CHUNK_SIZE = 10 * 1024 * 1024
 QUPATH_LAUNCH_MAX_WAIT_TIME = 30  # seconds, maximum wait time for QuPath to start
@@ -204,7 +208,7 @@ class Service(BaseService):
         """Initialize service."""
         super().__init__(Settings)
 
-    def info(self, mask_secrets: bool = True) -> dict[str, Any]:
+    def info(self, mask_secrets: bool = True) -> dict[str, Any]:  # noqa: ARG002, PLR6301
         """Determine info of this service.
 
         Args:
@@ -213,7 +217,6 @@ class Service(BaseService):
         Returns:
             dict[str,Any]: The info of this service.
         """
-        settings = self._settings.model_dump(context={UNHIDE_SENSITIVE_INFO: not mask_secrets})
         executable = Service.find_qupath_executable()
         version = Service.get_version()
         return {
@@ -221,11 +224,10 @@ class Service(BaseService):
                 "path": str(executable) if executable else None,
                 "version": dict(version) if version else None,
                 "expected_version": Service.get_expected_version(),
-            },
-            "settings": settings,
+            }
         }
 
-    def health(self) -> Health:
+    def health(self) -> Health:  # noqa: PLR6301
         """Determine health of this service.
 
         Returns:
@@ -233,36 +235,8 @@ class Service(BaseService):
         """
         return Health(
             status=Health.Code.UP,
-            components={
-                "application": self._determine_application_health(),
-            },
         )
 
-    @staticmethod
-    def _determine_application_health() -> Health:
-        """Determine we can reach a well known and secure endpoint.
-
-        - Checks if health endpoint is reachable and returns 200 OK
-        - Uses requests library for a direct connection check without authentication
-
-        Returns:
-            Health: The healthiness of the network connection via basic unauthenticated request.
-        """
-        try:
-            version = Service.get_version()
-            if not version:
-                message = "QuPath not installed."
-                return Health(status=Health.Code.DOWN, reason=message)
-            if version.version != Service.get_expected_version():
-                message = f"QuPath version mismatch: expected {QUPATH_VERSION}, got {version.version}"
-                logger.warning(message)
-                return Health(status=Health.Code.DOWN, reason=message)
-        except Exception as e:
-            message = f"Exception while checking health of QuPath application {e!s}"
-            logger.exception(message)
-            return Health(status=Health.Code.DOWN, reason=message)
-        return Health(status=Health.Code.UP)
-
     @staticmethod
     def _get_app_dir_from_qupath_dir(qupath_dir: Path, platform_system: str | None) -> Path:
         """Get the QuPath application directory based on the platform system.
@@ -333,10 +307,11 @@ class Service(BaseService):
                 text=True,
                 check=True,
                 timeout=QUPATH_LAUNCH_MAX_WAIT_TIME,
+                creationflags=SUBPROCESS_CREATION_FLAGS,
             )
 
             output = result.stdout.strip()
-            logger.debug("QuPath version output: %s", output)
+            logger.trace("QuPath version output: {}", output)
 
             version_match = re.search(r"Version:\s+([0-9]+\.[0-9]+\.[0-9]+(?:-rc[0-9]+)?)", output)
 
@@ -354,7 +329,7 @@ class Service(BaseService):
 
                 return QuPathVersion(version=version, build_time=build_time, commit_tag=commit_tag)
         except (subprocess.SubprocessError, subprocess.TimeoutExpired) as e:
-            logger.warning("Failed to get QuPath version from executable: %s", e)
+            logger.warning(f"Failed to get QuPath version from executable: {e}")
         except Exception:
             logger.exception("Unexpected error getting QuPath version")
 
@@ -428,7 +403,7 @@ class Service(BaseService):
 
         if not app_exe or not app_exe.is_file():
             if app_exe:
-                logger.warning("Expected to find file at %s.", app_exe)
+                logger.warning(f"Expected to find file at {app_exe}.")
             return None
 
         return app_exe
@@ -449,7 +424,7 @@ class Service(BaseService):
         Returns:
             Path: The directory QuPath will be installed into.
         """
-        return Path(appdirs.user_data_dir(__project_name__)).resolve()
+        return Path(platformdirs.user_data_dir(__project_name__)).resolve()
 
     @staticmethod
     def _download_qupath(  # noqa: C901, PLR0912, PLR0913, PLR0915, PLR0917
@@ -480,7 +455,7 @@ class Service(BaseService):
         """
         system = platform.system() if platform_system is None else platform_system
         machine = platform.machine() if platform_machine is None else platform_machine
-        logger.debug("Downloading QuPath version %s for system %s and machine %s", version, system, machine)
+        logger.trace("Downloading QuPath version {} for system {} and machine {}", version, system, machine)
 
         if system == "Linux":
             sys = "Linux"
@@ -513,7 +488,7 @@ class Service(BaseService):
 
         url = f"https://github.com/qupath/qupath/releases/download/{version}/{name}.{ext}"
 
-        logger.debug("Downloading QuPath from %s", url)
+        logger.trace("Downloading QuPath from {}", url)
 
         filename = Path(urlsplit(url).path).name
         filepath = path / filename
@@ -540,13 +515,13 @@ class Service(BaseService):
                                     archive_download_chunk_size=len(chunk),
                                 )
                                 install_progress_queue.put_nowait(progress)
-            logger.debug("Downloaded QuPath archive to '%s'", filepath)
+            logger.trace("Downloaded QuPath archive to '{}'", filepath)
         except requests.RequestException as e:
             message = f"Failed to download QuPath from {url}="
             logger.exception(message)
             raise RuntimeError(message) from e
         except Exception:
-            logger.exception("Error downloading QuPath from %s", url)
+            logger.exception(f"Error downloading QuPath from {url}")
             with contextlib.suppress(OSError):
                 filepath.unlink(missing_ok=True)
             raise
@@ -573,7 +548,7 @@ class Service(BaseService):
         """
         system = platform.system() if platform_system is None else platform_system
         machine = platform.machine() if platform_machine is None else platform_machine
-        logger.debug(
+        logger.trace(
             "Getting QuPath application directory for version '%s', installation path '%s' on system '%s'",
             version,
             installation_path,
@@ -625,7 +600,7 @@ class Service(BaseService):
             Path: The path to the extracted QuPath application directory.
         """
         system = platform.system() if platform_system is None else platform_system
-        logger.debug("Extracting QuPath archive '%s' to '%s' for system %s", archive_path, installation_path, system)
+        logger.trace("Extracting QuPath archive '{}' to '{}' for system {}", archive_path, installation_path, system)
 
         destination = Service.get_app_dir(
             version=QUPATH_VERSION,
@@ -702,6 +677,7 @@ class Service(BaseService):
                         command,
                         capture_output=True,
                         check=True,
+                        creationflags=SUBPROCESS_CREATION_FLAGS,
                     )
                 except subprocess.CalledProcessError as e:
                     stderr_output = e.stderr.decode("utf-8", errors="replace") if e.stderr else ""
@@ -729,12 +705,13 @@ class Service(BaseService):
                             f"cat '{payload_path.resolve()!s}' | gunzip -dc | cpio -i",
                         ]
                         if platform.system() == "Darwin"
-                        else ["7z", "x", str(payload_path.resolve()), "-o" + str(payload_extract_dir.resolve())]
+                        else ["7z", "x", str(payload_path.resolve()), f"-o{payload_extract_dir.resolve()!s}"]
                     )
                     subprocess.run(  # noqa: S603
                         command,
                         capture_output=True,
                         check=True,
+                        creationflags=SUBPROCESS_CREATION_FLAGS,
                     )
                 except subprocess.CalledProcessError as e:
                     stderr_output = e.stderr.decode("utf-8", errors="replace") if e.stderr else ""
@@ -829,7 +806,7 @@ class Service(BaseService):
                 install_progress_queue=progress_queue,
             )
             message = f"QuPath archive downloaded to '{archive_path!s}'."
-            logger.debug(message)
+            logger.trace(message)
 
             application_path = Service._extract_qupath(
                 archive_path=archive_path,
@@ -843,7 +820,7 @@ class Service(BaseService):
                 logger.error(message)
                 raise RuntimeError(message)  # noqa: TRY301
             message = f"QuPath application extracted to '{application_path!s}'."
-            logger.debug(message)
+            logger.trace(message)
 
             if extract_progress:
                 application_size = 0
@@ -851,7 +828,7 @@ class Service(BaseService):
                     if file_path.is_file():
                         application_size += file_path.stat().st_size
                 message = f"Total size of QuPath application: '{application_size}' bytes"
-                logger.debug(message)
+                logger.trace(message)
                 extract_progress(application_path, application_size=application_size)
 
             executable = Service.find_qupath_executable(
@@ -862,11 +839,11 @@ class Service(BaseService):
                 logger.error(message)
                 raise RuntimeError(message)  # noqa: TRY301
             message = f"QuPath executable found at '{executable!s}'."
-            logger.debug(message)
+            logger.trace(message)
 
             executable.chmod(0o755)  # Make sure the executable is runnable
             message = f"Set permissions set to 755 for QuPath executable at '{executable!s}'."
-            logger.debug(message)
+            logger.trace(message)
 
             return application_path
         except Exception as e:
@@ -902,7 +879,7 @@ class Service(BaseService):
             return None
 
         message = f"QuPath executable found at: {executable}"
-        logger.debug(message)
+        logger.trace(message)
 
         if platform.system() in {"Linux", "Darwin", "Windows"}:
             command = [str(executable)]
@@ -925,7 +902,7 @@ class Service(BaseService):
             raise NotImplementedError(message)
 
         try:
-            logger.debug("Launching QuPath with command: %s", " ".join(command))
+            logger.trace("Launching QuPath with command: {}", " ".join(command))
             process = subprocess.Popen(  # noqa: S603
                 command,
                 stdout=subprocess.PIPE,
@@ -956,7 +933,7 @@ class Service(BaseService):
                 if not output and exit_code is not None:
                     if script:
                         if exit_code == 0:
-                            logger.debug("QuPath script completed successfully.")
+                            logger.trace("QuPath script completed successfully.")
                             return process.pid
                         message = f"QuPath script failed with exit code '{exit_code}'."
                         logger.error(message)
@@ -966,9 +943,9 @@ class Service(BaseService):
                     break
 
                 if output:
-                    logger.debug(output.strip())
+                    logger.trace(output.strip())
                     if "qupath.lib.gui.QuPathApp - Starting QuPath with parameters" in output:
-                        logger.debug("QuPath started successfully.")
+                        logger.trace("QuPath started successfully.")
                         return process.pid
 
                 # Small sleep to prevent CPU hogging
@@ -1014,14 +991,14 @@ class Service(BaseService):
             try:
                 p.terminate()
             except psutil.NoSuchProcess:
-                logger.debug("Process %s already terminated.", p.pid)
+                logger.trace("Process {} already terminated.", p.pid)
                 continue
         _, alive = wait_procs(procs, timeout=wait_before_kill)
         for p in alive:
             try:
                 p.kill()
             except psutil.NoSuchProcess:
-                logger.debug("Process %s already terminated.", p.pid)
+                logger.trace("Process {} already terminated.", p.pid)
                 continue
         return len(procs)
 
@@ -1060,13 +1037,13 @@ class Service(BaseService):
             removed = False
             for qupath in path.glob("QuPath*"):
                 if qupath.is_dir():
-                    logger.debug("Removing QuPath directory '%s'", qupath)
+                    logger.trace("Removing QuPath directory '{}'", qupath)
                 else:
-                    logger.debug("Removing QuPath archive '%s'", qupath)
+                    logger.trace("Removing QuPath archive '{}'", qupath)
                 try:
                     shutil.rmtree(qupath, ignore_errors=False)
                     removed = True
-                except Exception as e:  # noqa: BLE001
+                except Exception as e:
                     message = f"Failed to remove '{qupath!s}': {e!s}"
                     logger.warning(message)
             return removed
@@ -1086,10 +1063,10 @@ class Service(BaseService):
             logger.error(message)
             raise ValueError(message)
         try:
-            logger.debug("Removing '%s'", app_dir)
+            logger.trace("Removing '{}'", app_dir)
             shutil.rmtree(app_dir, ignore_errors=False)
             return True
-        except Exception as e:  # noqa: BLE001
+        except Exception as e:
             message = f"Failed to remove '{app_dir!s}': {e!s}"
             logger.warning(message)
         return False
@@ -1136,7 +1113,7 @@ class Service(BaseService):
             RuntimeError: If there is an unexpected error adding images to the project.
         """
         message = f"Adding images to QuPath project at '{project!s}' from paths: {paths!r}"
-        logger.debug(message)
+        logger.trace(message)
 
         if progress_callable:
             progress = AddProgress()
@@ -1170,7 +1147,7 @@ class Service(BaseService):
                 added_count = int(result_data.get("added_count", 0))
                 errors = result_data.get("errors", [])
                 for error in errors:
-                    logger.warning("QuPath add script error: %s", error)
+                    logger.warning(f"QuPath add script error: {error}")
 
                 if progress_callable:
                     progress.status = AddProgressState.COMPLETED
